/*
  LiquidCrystal Library - Hello World

 Demonstrates the use a 16x2 LCD display.  The LiquidCrystal
 library works with all LCD displays that are compatible with the
 Hitachi HD44780 driver. There are many of them out there, and you
 can usually tell them by the 16-pin interface.

 This sketch prints "Hello World!" to the LCD
 and shows the time.

  The circuit:
 * LCD RS pin to digital pin 12
 * LCD Enable pin to digital pin 11
 * LCD D4 pin to digital pin 5
 * LCD D5 pin to digital pin 4
 * LCD D6 pin to digital pin 3
 * LCD D7 pin to digital pin 2
 * LCD R/W pin to ground
 * LCD VSS pin to ground
 * LCD VCC pin to 5V
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)

 Library originally added 18 Apr 2008
 by David A. Mellis
 library modified 5 Jul 2009
 by Limor Fried (http://www.ladyada.net)
 example added 9 Jul 2009
 by Tom Igoe
 modified 22 Nov 2010
 by Tom Igoe

 This example code is in the public domain.

 http://www.arduino.cc/en/Tutorial/LiquidCrystal
 */

// include the library code:
#include <LiquidCrystal.h>

//  original initializer
// initialize the library with the numbers of the interface pins
//LiquidCrystal lcd(12, 11, 5, 4, 3, 2);


//CHECK IF LCD SHIELD USES DIGITAL PIN 10 AS THIS IS USED AS CS FOR DATA LOGGING SHIELD
//CHECK TO SEE IF LCD IS ONE OF THE FAULTY ONES
//MINE HAD BOTH PROBLEMS SO I DESOLDERED AND REMOVED THE FAULTY PIN 10
//REFERENCE - https://forum.arduino.cc/index.php?topic=96747.0 (Retrieved 7/June/2019)

//new initializer
// select the pins used on the LCD panel
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
// http://www.robotshop.com/content/PDF/dfrobot-lcd-keypad-shield-schematic.pdf 



const int NUMBER_OF_ELEMENTS = 110;
const int MAX_SIZE = 4;

//Official Scrabble (TM) Words list

const char descriptions [NUMBER_OF_ELEMENTS] [MAX_SIZE] PROGMEM = { 
 {"AA"}, 
 {"AB"}, 
 {"AC"}, 
 {"AD"}, 
 {"AE"}, 
 {"AG"}, 
 {"AH"}, 
 {"AI"}, 
 {"AL"}, 
 {"AM"},
 {"AN"}, 
 {"AR"},
 {"AS"},
 {"AT"},
 {"AW"},
 {"AX"},
 {"AY"},//17

 {"BA"},
 {"BE"},
 {"BI"}, 
 {"BO"},
 {"BY"},
 {"DA"},
 {"DE"},
 {"DO"},//25

 {"ED"}, 
 {"EF"},
 {"EH"},
 {"EL"},
 {"EM"},
 {"EN"},
 {"ER"},
 {"ES"},
 {"ET"},
 {"EW"},
 {"EX"},//36
 
 {"FA"},
 {"FE"},//38
 
 {"GI"},
 {"GO"},//40

 {"HA"},
 {"HE"},
 {"HI"},
 {"HM"},
 {"HO"},//45

 {"ID"},
 {"IF"},
 {"IN"},
 {"IS"},
 {"IT"},
 {"IE"},//51
 
 {"JO"},//52
 
 {"KA"},
 {"KI"},//54
  
 {"LA"},
 {"LI"},
 {"LO"},//57

 {"MA"},
 {"ME"},
 {"MI"},
 {"MM"},
 {"MO"},
 {"MU"},
 {"MY"},//64
 
 {"NA"},
 {"NE"},
 {"NO"},
 {"NU"},//68

 {"OD"},
 {"OE"},
 {"OF"},
 {"OH"},
 {"OI"},
 {"OK"},
 {"OL"},
 {"OM"},
 {"ON"},
 {"OP"},
 {"OR"},
 {"OS"},
 {"OW"},
 {"OX"},
 {"OY"},//83

 {"PA"},
 {"PE"},
 {"PI"},
 {"PO"},//87
   
 {"QI"},//88
 
 {"RE"},//89
 
 {"SH"},
 {"SI"},
 {"SO"},//92
 
 {"TA"},
 {"TE"},
 {"TI"},
 {"TO"},//96

 {"UH"},
 {"UM"},
 {"UN"},
 {"UP"},
 {"US"},
 {"UT"},//102
  
 {"WO"},
 {"WE"},//104
 
 {"XI"},
 {"XU"},//106
 
 {"YA"},
 {"YE"},
 {"YO"},
 {"ZA"},//110
  
 };

const int sounder_pin=3;// the pin we attach our sounder to
const int frequency=1000;


void setup() {
  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  // Print a message to the LCD.
  lcd.clear();

  // initialize digital pin as an output.
  pinMode(sounder_pin, OUTPUT);
}

void loop() {
  
  randomSeed(analogRead(1));
  
    // make a copy of the current one

    int x=random(0,NUMBER_OF_ELEMENTS);
    
    char oneItem [MAX_SIZE];
    memcpy_P (&oneItem, &descriptions [x], sizeof oneItem);
    
    lcd.setCursor(6,0);
    lcd.write (oneItem);
    
  	beep_character(toLowerCase(oneItem[0]),60);
    beep_character(toLowerCase(oneItem[1]),60);
  	
    delay(1500);
}

//morse code subroutine

//beep_character function to sound out morse
//uses functions delay() and analogWrite()
//all timing is based around the length of the dot
//source for timings -> http://www.kent-engineers.com/codespeed.htm (retreived 5 March 2017)
//examples
//unit_length = 240, dot time calculated for 5 words per minute
//unit_length = 92, dot time calculated for 13 words per minute
//unit_length = 60, dot time calculated for 20 words per minute

void beep_character(int incoming_byte, unsigned long unit_length)
{

//here the standard morse dichotic tree has been modified to allow direct translation from character look up to dots and dashes
//by rearranging the tree we eliminate the need to use a stack  
const int reverse_morse_array[]={
0x00,0x00,//positions 0 and 1 not used
101,116,//2
105,110,97,109,//4
115,100,114,103,117,107,119,111,//8
104,98,108,122,102,99,112,246,118,120,228,113,252,121,106,154,//16
53,54,0x00,55,233,231,254,56,83,47,43,71,208,72,74,57,52,61,232,0x00,13,0x00,224,0x00,51,0x00,71,241,50,0x00,49,48,//32
0x00,0x00,0x00,0x00,0x00,0x00,0x00,58,0x00,0x00,0x00,0x00,63,95,0x00,0x00,0x00,0x00,34,0x00,0x00,59,64,0x00,0x00,0x00,64,0x00,0x00,40,39,
0x00,0x00,45,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,46,33,95,0x00,40,0x00,0x00,0x00,0x00,44,0x00,33,0x00,0x00,58,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
const int reverse_morse_array_limit=127;//128 elements total so 0 to 127 

const unsigned long dot=unit_length;
const unsigned long dash=unit_length*3;
const unsigned long intra_letter_space=unit_length;//space between dot's and dashe's within letter
const unsigned long inter_letter_space=(unit_length*3)-intra_letter_space;// space between letters, subtracts used to deal with spaces already there
const unsigned long word_space=(unit_length*7)-intra_letter_space-inter_letter_space;// space between words, subtracts used to deal with spaces already there

int i=0;

if (incoming_byte==' '){delay(word_space);return;}
 

//find character in array
    
for(i=2;i<=reverse_morse_array_limit;i++)
{
  if (incoming_byte==reverse_morse_array[i]){break;}//locate letter
  }

if (i<=reverse_morse_array_limit){
   
  //work backwards -> stopping at arrray position 1
  //odd or even -> with binary number first bit always set if odd
  //so bitwise & index with 1, if result = 1 then index is odd 
  //if odd prior position = (current position -1)/2  
  //if even prior position = current position/2  
  //sound dash for odd, dot for even
  //repeat until you reach the root of binary tree, that is position 1

  while((i!=1))
            {
              if (i&1){ //odd so dash
                        tone(sounder_pin,frequency);
                        delay(dash);                       // wait for a while
                        noTone(sounder_pin);
                        delay(intra_letter_space);         // space between letter part                     
                		i=(i-1)/2; }
                      else{ //even so dot
                        	tone(sounder_pin,frequency);
                            delay(dot);                    // wait for a while  
                        	noTone(sounder_pin);
                            delay(intra_letter_space);     // space between letter part
                        	i=i/2;}
              }   delay(inter_letter_space);//space between letters                                         
                              }

  } 

 
